<?php 

namespace Teknisa\MAN\Controllers;

use Zeedhi\Framework\DTO;
use Zeedhi\Framework\Report\ReportService;
use Zeedhi\Framework\Remote\cURLRequest;
use Zeedhi\Framework\Remote\HttpInterface;

class BirtReport extends \Zeedhi\Framework\Controller\Report {
    
    protected $path;
    protected $dns;
    protected $root;
    protected $reportMapping = array (
        'rfc' => array (
            'strategy' => 'BIRT',
            'parameterMapping' => array (
                "CDFILIAL" => "CDFILIAL",
                "CDCAIXA" => "CDCAIXA",
                "NRORG" => "NRORG",
                "DATAINICIO" => "DATAINICIO",
                "DATAFINAL" => "DATAFINAL",
                "DTENTRVENDA" => "DTENTRVENDA"
            )
        ),
        'rfc_ecf' => array (
            'strategy' => 'BIRT',
            'parameterMapping' => array (
                "CDFILIAL" => "CDFILIAL",
                "CDCAIXA" => "CDCAIXA",
                "NRORG" => "NRORG",
                "DATAINICIO" => "DATAINICIO",
                "DATAFINAL" => "DATAFINAL",
                "DTENTRVENDA" => "DTENTRVENDA"
            )
        ),
        'rmc' => array (
            'strategy' => 'BIRT',
            'parameterMapping' => array (
                "CDFILIAL" => "CDFILIAL",
                "CDCAIXA" => "CDCAIXA",
                "NRORG" => "NRORG",
                "DATA" => "DATA",
                "DATES" => "DATES",
                "FILIAIS" => "FILIAIS", 
                "DATA_REPEATER" => "DATA_REPEATER",
                "FILIAL_REPEATER" => "FILIAL_REPEATER",
                "CAIXA_REPEATER" => "CAIXA_REPEATER",
                "CAIXA_POSICAO" => "CAIXA_POSICAO"
            )
        ),
        'rmcns' => array (
            'strategy' => 'BIRT',
            'parameterMapping' => array (
                "CDFILIAL" => "CDFILIAL",
                "CDCAIXA" => "CDCAIXA",
                "NRORG" => "NRORG",
                "DATA" => "DATA",
                "DATES" => "DATES",
                "FILIAIS" => "FILIAIS", 
                "DATA_REPEATER" => "DATA_REPEATER",
                "FILIAL_REPEATER" => "FILIAL_REPEATER",
                "CAIXA_REPEATER" => "CAIXA_REPEATER",
                "CAIXA_POSICAO" => "CAIXA_POSICAO"
            )
        ),
        'vendaitemcancperiodo' => array (
            'strategy' => 'BIRT',
            'parameterMapping' => array (
                "CDFILIAL" => "CDFILIAL",
                "CDCAIXA" => "CDCAIXA",
                "TCDCAIXA" => "TCDCAIXA",
                "NRORG" => "NRORG",
                "DTVENDAINI" => "DTVENDAINI",
                "DTVENDAFIN" => "DTVENDAFIN",
                "CDCLIENTE" => "CDCLIENTE",
                "TCDCLIENTE" => "TCDCLIENTE",
                "CDCONSUMIDOR" => "CDCONSUMIDOR",
                "TCDCONSUMIDOR" => "TCDCONSUMIDOR",
                "CDVENDEDOR" => "CDVENDEDOR",
                "TCDVENDEDOR" => "TCDVENDEDOR",
                "CDPICTPROD" => "CDPICTPROD"
            )
        )
        
    );
    
    public function __construct(ReportService $reportService, $path, $dns) {
        $this->path = $path;
        $this->dns  = $dns;
        $root = getcwd();
        $root = str_replace('management/modules/man/backend/service', '', $root);
        $this->root = $this->getPath($root);
        parent::__construct($reportService);
    }
    
    /*
    *  Sobresceve rotina geração de link do relatórios do zeedhi.
    *  Retorna o link de um html temporario que abre o pdf do relatório
    *  e exclui automitacamente após ser aberto.
    *  
    *  @param DTO\Request\Row $request Requisição dom dados do relatório
    *  @param DTO\Response    $response Response onde será retornado os links
    *  @param string          $parentMethod Define se a rotina utilizada pelo 
    *  relatório vai ser a do zeedhi ou do Controller. Default = Controller
    *
    */
    public function createRemoteReport(DTO\Request\Row $request, DTO\Response $response, $parentMethod = false) {
		try {
		    $row = $request->getRow();
            $reportName = $this->getReportName($row);
            
		    if($parentMethod) parent::createRemoteReport($request, $response); 
		    else {
                $this->beforeProcessReport($request, $response);
                $reportURL = $this->processReport($request);
    			$this->afterProcessReport($request, $response, $reportURL);
    			
    			$reportURL = str_replace('frameset', 'preview', $reportURL);

    	        $index = $this->dns.'management/modules/man/backend/service';
    	        
                $hash = uniqid();
                $savePath = $this->root.$this->path.'/view/'.$reportName.'_'.$hash;
                $linkPath = str_replace($this->root, $this->dns, $savePath);
                $savePath = $this->getPath($savePath);
                
                $report = $this->getReport($reportURL);
                
                $reportScript = '<html style="overflow-y:hidden">
                <head>
                    <iframe src="'.$linkPath.'.pdf" width="101%" height="101%" style="margin: -10px; border: none;"></iframe>
                    <script src="http://ajax.googleapis.com/ajax/libs/jquery/1.3/jquery.min.js" type="text/javascript"></script>
                    <script type="text/javascript">
                      $(document).ready(function() {
                         var data = {reportName: "'.$reportName.'_'.$hash.'"};
                         $.ajax({
                             url:"'.$index.'/index.php/renderReport",
                             type: "POST",
                             data: data,
                             dataType: "JSON"
                         });
                      });
                     </script>
               </head>
            </html>';
            
                file_put_contents($savePath.'.pdf', $report);
                file_put_contents($savePath.'.html', $reportScript);
                
                $linkReport = $this->dns.$savePath;
            
    			$response->addMessage(new DTO\Response\Message($linkPath.'.html'));
		    }
			
		} catch (\Exception $e) {
			$response->setError(new DTO\Response\Error($e->getMessage(), $e->getCode(), $e->getTraceAsString()));
		}
	}
    /*
    *  Gera o relatório no servidor e retorna a string do pdf ou html do 
    *  relatório.
    *
    *  @param string $reportName Nome do relatório
  	*  @param array  $params Array com os parâmetros do relatório.
  	*  @param string $format Formato em que o relatório será gerado.
  	*  
  	*  return string $report 
    */
    public function getReport($reportUrl) {
        try {
            $reportUrl = str_replace('frameset', 'preview', $reportUrl);
            
            $cURLRequest = new cURLRequest($reportUrl);

            $report = $cURLRequest->request('');
            
            $requestInfo = $cURLRequest->getRequestInfo();
            if($requestInfo['http_code'] !== HTTPInterface::STATUS_CODE_OK) {
                throw new \Exception('Servidor de emissão de relatórios indisponível. Contate o administrador de rede');
            }
            else return $report;
          }
        catch (\Exception $e) {
            throw $e;
        }
    }
    
    /*
    *  Função chamada ao abrir o  html gerado no metódo openReport. 
    *  Remove os arquivos pdf e html tempórarios do servidor.
    *  
    *  @param DTO\Request $request Requisição feita pelo html com nome do relatório.
    *  @param DTO\Response $response
    */
    public function renderReport(DTO\Request $request, DTO\Response $response) {
        $reportName = $request->getParameters()["reportName"];
        
        $deletePath = $this->root.$this->path.'/view/'.$reportName;
        
        if(file_exists($deletePath.'.pdf')) $this->deleteReport($deletePath.'.pdf');
        if(file_exists($deletePath.'.html')) $this->deleteReport($deletePath.'.html');
    }
    
    /*
    *  Remove o relatório do servidor através de um diretório.
    *   
    *  @param string $deletePath diretório onde o relatório se encontra
    * 
    *  return boolean 
    */
    public function deleteReport($deletePath) {
        if(file_exists($deletePath)) {
          return unlink($deletePath);
        }
        else return false;
    }
    
    /*
    *  Recebe um diretório e muda o separador('\\', '//', '/') 
    *  para o do sistema operacional.
    *  
    *  @param string $path diretório
    *  
    *  return string diretório formatado
    */
    private function getPath($path) {
        return str_replace(['\\', '//', '/'], DIRECTORY_SEPARATOR, $path);
    }
    
}